/* 
 * File:   gdgnet.h
 * Author: Michal Hucik <hucik@ordoz.com>
 *
 * Created on 13. května 2019, 21:45
 * 
 * 
 * ----------------------------- License -------------------------------------
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * ---------------------------------------------------------------------------
 */


#ifndef GDGNET_H
#define GDGNET_H

#ifdef __cplusplus
extern "C" {
#endif

#include <glib.h>
#include "gdgelement.h"

#define GDGNET_PRINT_NAME_PREFIX "net_"

    typedef enum en_GDGNET_REALPIN_TYPE {
        GDGNET_REALPIN_TYPE_UNKNOWN = 0,
        GDGNET_REALPIN_TYPE_IN, // pin je opravdu vstupni :)
        GDGNET_REALPIN_TYPE_OUT,
    } en_GDGNET_REALPIN_TYPE;


    typedef struct st_GDGNET_PIN {
        st_GDGELEMENT *element; /* pokud je NULL, tak se jedna o PIN */
        en_GDGNET_REALPIN_TYPE rptype; /* pokud jsme pripojeni na skutecny IO pin, tak je zde jeho typ */
        char *part_pin_name;
        void *parent_net;
        int loc_x;
        int loc_y;
        int loc_z; /* pokud je u PIN uvedena hodnota 'z', tak se jedna o vnejsi GDG pin */
    } st_GDGNET_PIN;


    typedef struct st_GDGNET {
        char *name;
        gboolean is_constant;
        gboolean c_value;
        int count_pin;
        st_GDGNET_PIN **pin;
        int count_output; // v signalech mezi party s vyjimkou vdd a 0 musi byt vzdy '1', v PIN signalech nam identifikuje pin direction
        int count_externals; // pocet PIN pripojeni
    } st_GDGNET;

    typedef enum en_GDGNETSALL_TYPE {
        GDGNETSALL_TYPE_MASTER = 0,
        GDGNETSALL_TYPE_SLAVE,
    } en_GDGNETSALL_TYPE;

    typedef struct st_GDGNETSALL {
        en_GDGNETSALL_TYPE type;
        int count_net;
        st_GDGNET **net;
    } st_GDGNETSALL;


    typedef struct st_GDGIOPORT {
        st_GDGNET_PIN *pin;
        st_GDGNET *net;
        char *vhdl_name;
    } st_GDGIOPORT;


    typedef struct st_GDGIOPORTSALL {
        int count_real;
        st_GDGIOPORT **real;
        int count_internal;
        st_GDGIOPORT **internal;
        int count_opennet;
        st_GDGIOPORT **opennet;
    } st_GDGIOPORTSALL;


    extern st_GDGNETSALL* gdgnets_init ( st_GDGELEMENTSALL *elmall, const char * filepath );
    extern st_GDGNETSALL* gdgnets_create_slave_from_nets ( st_GDGNET **nets, int nets_count );
    extern void gdgnets_destroy ( st_GDGNETSALL *netall );
    extern void gdgnet_pin_destroy ( st_GDGNET_PIN *pin );
    extern void gdgnet_remove_part_pin ( st_GDGNET *net, st_GDGELEMENT *elm, const char *part_pin_name, int loc_x, int loc_y, int loc_z );
    extern void gdgnet_merge ( st_GDGNET *dstnet, st_GDGNET *srcnet );
    extern void gdgnets_remove_net ( st_GDGNETSALL *netall, st_GDGNET *remove_net );
    
    extern st_GDGNET* gdgnets_get_net_by_name ( st_GDGNETSALL *netall, const char *name );
    extern st_GDGNET_PIN* gdgnet_get_net_source_pin ( st_GDGNET *net );

    extern st_GDGIOPORTSALL* gdgnets_create_ioports ( st_GDGNETSALL *netall, void *partall_p );
    extern void gdgnets_ioports_destroy ( st_GDGIOPORTSALL *ioportall );


#ifdef __cplusplus
}
#endif

#endif /* GDGNET_H */

