/* 
 * File:   gdgpath.h
 * Author: Michal Hucik <hucik@ordoz.com>
 *
 * Created on 21. května 2019, 11:19
 * 
 * 
 * ----------------------------- License -------------------------------------
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * ---------------------------------------------------------------------------
 */


#ifndef GDGPATH_H
#define GDGPATH_H

#ifdef __cplusplus
extern "C" {
#endif

#include <glib.h>
#include "gdgpart.h"
#include "gdgnet.h"


    typedef struct st_GDGPATH {
        st_GDGNET *dst_net;
        int net_count;
        int pin_count;
        int part_count;
        int element_count;
        st_GDGNET_PIN **pin;
        st_GDGNET **net;
        st_GDGPART **part;
        st_GDGELEMENT **element;

        st_GDGPARTSALL *partall;
        gboolean create_path_str;
        gboolean verbose;
        GString *str;
    } st_GDGPATH;

    extern st_GDGPATH* gdgpath_new ( st_GDGPARTSALL *partall );
    extern void gdgpath_destroy ( st_GDGPATH *path );

    extern st_GDGPATH* gdgpath_create_for_net ( st_GDGPARTSALL *partall, st_GDGNET *net, gboolean create_path_str, gboolean verbose );

    extern int gdgpath_path_has_net ( st_GDGPATH *path, st_GDGNET *net );
    extern int gdgpath_path_has_pin ( st_GDGPATH *path, st_GDGNET_PIN *pin );
    extern int gdgpath_path_has_part ( st_GDGPATH *path, st_GDGPART *part );
    extern int gdgpath_path_has_element ( st_GDGPATH *path, st_GDGELEMENT *elm );

    extern gboolean gdgpath_add_net ( st_GDGPATH *path, st_GDGNET *net );
    extern void gdgpath_add_pin ( st_GDGPATH *path, st_GDGNET_PIN *pin );
    extern gboolean gdgpath_add_part ( st_GDGPATH *path, st_GDGPART *part );
    extern void gdgpath_add_element ( st_GDGPATH *path, st_GDGELEMENT *elm );

    extern char* gdgpath_get_elements_txt ( st_GDGPATH *path );
    extern char* gdgpath_get_vhdl ( st_GDGPATH *path, gboolean optimalized_output );

#ifdef __cplusplus
}
#endif

#endif /* GDGPATH_H */

